local MakePlayerCharacter = require "prefabs/player_common"

local assets = {
        Asset( "ANIM", "anim/player_basic.zip" ),
        Asset( "ANIM", "anim/player_idles_shiver.zip" ),
        Asset( "ANIM", "anim/player_actions.zip" ),
        Asset( "ANIM", "anim/player_actions_axe.zip" ),
        Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
        Asset( "ANIM", "anim/player_actions_shovel.zip" ),
        Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
        Asset( "ANIM", "anim/player_actions_eat.zip" ),
        Asset( "ANIM", "anim/player_actions_item.zip" ),
        Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
        Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
        Asset( "ANIM", "anim/player_actions_fishing.zip" ),
        Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
        Asset( "ANIM", "anim/player_bush_hat.zip" ),
        Asset( "ANIM", "anim/player_attacks.zip" ),
        Asset( "ANIM", "anim/player_idles.zip" ),
        Asset( "ANIM", "anim/player_rebirth.zip" ),
        Asset( "ANIM", "anim/player_jump.zip" ),
        Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
        Asset( "ANIM", "anim/player_teleport.zip" ),
        Asset( "ANIM", "anim/wilson_fx.zip" ),
        Asset( "ANIM", "anim/player_one_man_band.zip" ),
        Asset( "ANIM", "anim/shadow_hands.zip" ),
        Asset( "SOUND", "sound/sfx.fsb" ),
        Asset( "SOUND", "sound/wilson.fsb" ),
        Asset( "ANIM", "anim/beard.zip" ),

        Asset( "ANIM", "anim/wayland.zip" ),
}
local prefabs = {}
local start_inv = {
	"monstermeat",
	"nightmarefuel",
}

-- When loading or spawning the character
local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
    inst:ListenForEvent("ms_becameghost", onbecameghost)

    if inst:HasTag("playerghost") then
        onbecameghost(inst)
    else
        onbecamehuman(inst)
    end
end

-- despawn pet
local function OnDespawn(inst, data)
	for k,v in pairs(inst.components.leader.followers) do
		if k.prefab == "terrorbeakpet" then
			k:Remove() 
		end
	end
end

-- spawn pet
local function SpawnTerrorbeakpet(player)
	if player:HasTag("playerghost") then return false end
	
	for k,v in pairs(player.components.leader.followers) do
		if k.prefab=="terrorbeakpet" then return false end
	end

    local pos = Vector3(player.Transform:GetWorldPosition())
    local offset = (FindWalkableOffset(pos,math.random()*PI*2,0.5,false))
	
    if offset == nil then
          if  player.components.talker then
			player.components.talker:Say("No Space!", 2.5)
          end
          return false
    end
    pos=pos+offset
    local unit = SpawnPrefab("terrorbeakpet")
    
	unit.Transform:SetPosition (pos:Get())
	player.components.leader:AddFollower(unit) 
	unit.spawnedforplayer=player
	end

-- function activates on health data
local function OnHealthDelta(inst, data)
	if data.newpercent <= .99 then 
		inst:DoTaskInTime(0, function() SpawnTerrorbeakpet(inst) end)
	end
end

local fn = function(inst)
	-- SPEECH
	inst.soundsname = "maxwell"

	-- STATS	
	inst.components.health:SetMaxHealth(99)
	inst.components.hunger:SetMax(99)
	inst.components.sanity:SetMax(99)
	
	-- MONSTER EATER
	inst.components.eater.strongstomach = true
	inst.components.eater.monsterimmune = true
	
	--MOVEMENT MODIFICATIONS
		-- walk speed
	inst.components.locomotor.walkspeed = 4.7
		-- run speed
	inst.components.locomotor.runspeed = 6.4
	
	--SANITY MODIFICATIONS
		-- sanity loss around enemies
	inst.components.sanity.neg_aura_mult = .5
		-- sanity loss at night
	inst.components.sanity.night_drain_mult = 0
		-- sanity gain passively
	inst.components.sanity.dapperness = .01
	
	--BUILDER MODIFICATIONS
		-- default magic craftability
	inst.components.builder.magic_bonus = 2
		-- default ancient craftability
	inst.components.builder.ancient_bonus = 2
	
	-- Damage multiplier (optional)
    inst.components.combat.damagemultiplier = .23
	
	-- Hunger rate (optional)
	inst.components.hunger.hungerrate = .5 * TUNING.WILSON_HUNGER_RATE
	
	--PET MODIFICATIONS
		---Spawn pet when health is low
	inst.SpawnTerrorbeakpet = SpawnTerrorbeakpet
	inst:ListenForEvent("healthdelta", OnHealthDelta)
	
	--FOOD SANITY GAIN
		-- gain sanity from monster meat
	local OldEat = inst.components.eater.Eat
		inst.components.eater.Eat = function(self, food)
		if food and food.components.edible and food.prefab == "monstermeat" then
				food.components.edible.sanityvalue = 13
		end	
		return OldEat(self, food)
	end
		-- gain sanity from cooked monster meat
	local OldEat = inst.components.eater.Eat
		inst.components.eater.Eat = function(self, food)
		if food and food.components.edible and food.prefab == "cookedmonstermeat" then
				food.components.edible.sanityvalue = 13
				food.components.edible.healthvalue = 13
		end	
		return OldEat(self, food)
	end
	
		-- gain sanity from cooked monster meat
	local OldEat = inst.components.eater.Eat
		inst.components.eater.Eat = function(self, food)
		if food and food.components.edible and food.prefab == "monsterlasagna" then
				food.components.edible.sanityvalue = 13
				food.components.edible.healthvalue = 13
		end	
		return OldEat(self, food)
	end
	
		-- gain sanity from cooked monster meat
	local OldEat = inst.components.eater.Eat
		inst.components.eater.Eat = function(self, food)
		if food and food.components.edible and food.prefab == "monstermeat_dried" then
				food.components.edible.sanityvalue = 13
				food.components.edible.healthvalue = 13
		end	
		return OldEat(self, food)
	end
	
end

return MakePlayerCharacter("wayland", prefabs, assets, fn, start_inv)
